kedro.extras.datasets.pandas.excel_dataset — Kedro 0.17.6 documentation

# Source code for kedro.extras.datasets.pandas.excel_dataset

    """``ExcelDataSet`` loads/saves data from/to a Excel file using an underlying
    filesystem (e.g.: local, S3, GCS). It uses pandas to handle the Excel file.
    """
    from copy import deepcopy
    from io import BytesIO
    from pathlib import PurePosixPath
    from typing import Any, Dict, Union

    import fsspec
    import pandas as pd

    from kedro.io.core import (
        AbstractVersionedDataSet,
        DataSetError,
        Version,
        get_filepath_str,
        get_protocol_and_path,
    )

    class ExcelDataSet(AbstractVersionedDataSet):
        """``ExcelDataSet`` loads/saves data from/to a Excel file using an underlying
        filesystem (e.g.: local, S3, GCS). It uses pandas to handle the Excel file.

        Example adding a catalog entry with
        `YAML API <https://kedro.readthedocs.io/en/stable/05_data/\
            01_data_catalog.html#using-the-data-catalog-with-the-yaml-api>`_:

        .. code-block:: yaml

            >>> rockets:
            >>>   type: pandas.ExcelDataSet
            >>>   filepath: gcs://your_bucket/rockets.xlsx
            >>>   fs_args:
            >>>     project: my-project
            >>>   credentials: my_gcp_credentials
            >>>   save_args:
            >>>     sheet_name: Sheet1
            >>>   load_args:
            >>>     sheet_name: Sheet1
            >>>
            >>> shuttles:
            >>>   type: pandas.ExcelDataSet
            >>>   filepath: data/01_raw/shuttles.xlsx

        Example using Python API:
        ::

            >>> from kedro.extras.datasets.pandas import ExcelDataSet
            >>> import pandas as pd
            >>>
            >>> data = pd.DataFrame({'col1': [1, 2], 'col2': [4, 5],
            >>>                      'col3': [5, 6]})
            >>>
            >>> # data_set = ExcelDataSet(filepath="gcs://bucket/test.xlsx")
            >>> data_set = ExcelDataSet(filepath="test.xlsx")
            >>> data_set.save(data)
            >>> reloaded = data_set.load()
            >>> assert data.equals(reloaded)

        """

        DEFAULT_LOAD_ARGS = {"engine": "xlrd"}
        DEFAULT_SAVE_ARGS = {"index": False}

        # pylint: disable=too-many-arguments
        def __init__(
            self,
            filepath: str,
            engine: str = "xlsxwriter",
            load_args: Dict[str, Any] = None,
            save_args: Dict[str, Any] = None,
            version: Version = None,
            credentials: Dict[str, Any] = None,
            fs_args: Dict[str, Any] = None,
        ) -> None:
            """Creates a new instance of ``ExcelDataSet`` pointing to a concrete Excel file
            on a specific filesystem.

            Args:
                filepath: Filepath in POSIX format to a Excel file prefixed with a protocol like
                    `s3://`. If prefix is not provided, `file` protocol (local filesystem) will be used.
                    The prefix should be any protocol supported by ``fsspec``.
                    Note: `http(s)` doesn't support versioning.
                engine: The engine used to write to excel files. The default
                    engine is 'xlsxwriter'.
                load_args: Pandas options for loading Excel files.
                    Here you can find all available arguments:
                    https://pandas.pydata.org/pandas-docs/stable/generated/pandas.read_excel.html
                    All defaults are preserved, but "engine", which is set to "xlrd".
                    Supports multi-sheet Excel files (include `sheet_name = None` in `load_args`).
                save_args: Pandas options for saving Excel files.
                    Here you can find all available arguments:
                    https://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.to_excel.html
                    All defaults are preserved, but "index", which is set to False.
                    If you would like to specify options for the `ExcelWriter`,
                    you can include them under the "writer" key. Here you can
                    find all available arguments:
                    https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.ExcelWriter.html
                version: If specified, should be an instance of
                    ``kedro.io.core.Version``. If its ``load`` attribute is
                    None, the latest version will be loaded. If its ``save``
                    attribute is None, save version will be autogenerated.
                credentials: Credentials required to get access to the underlying filesystem.
                    E.g. for ``GCSFileSystem`` it should look like `{"token": None}`.
                fs_args: Extra arguments to pass into underlying filesystem class constructor
                    (e.g. `{"project": "my-project"}` for ``GCSFileSystem``), as well as
                    to pass to the filesystem's `open` method through nested keys
                    `open_args_load` and `open_args_save`.
                    Here you can find all available arguments for `open`:
                    https://filesystem-spec.readthedocs.io/en/latest/api.html#fsspec.spec.AbstractFileSystem.open
                    All defaults are preserved, except `mode`, which is set to `wb` when saving.
            """
            _fs_args = deepcopy(fs_args) or {}
            _fs_open_args_load = _fs_args.pop("open_args_load", {})
            _fs_open_args_save = _fs_args.pop("open_args_save", {})
            _credentials = deepcopy(credentials) or {}

            protocol, path = get_protocol_and_path(filepath, version)
            if protocol == "file":
                _fs_args.setdefault("auto_mkdir", True)

            self._protocol = protocol
            self._fs = fsspec.filesystem(self._protocol, **_credentials, **_fs_args)

            super().__init__(
                filepath=PurePosixPath(path),
                version=version,
                exists_function=self._fs.exists,
                glob_function=self._fs.glob,
            )

            # Handle default load arguments
            self._load_args = deepcopy(self.DEFAULT_LOAD_ARGS)
            if load_args is not None:
                self._load_args.update(load_args)

            # Handle default save arguments
            self._save_args = deepcopy(self.DEFAULT_SAVE_ARGS)
            if save_args is not None:
                self._save_args.update(save_args)
            self._writer_args = self._save_args.pop("writer", {"engine": engine})

            _fs_open_args_save.setdefault("mode", "wb")
            self._fs_open_args_load = _fs_open_args_load
            self._fs_open_args_save = _fs_open_args_save

        def _describe(self) -> Dict[str, Any]:
            return dict(
                filepath=self._filepath,
                protocol=self._protocol,
                load_args=self._load_args,
                save_args=self._save_args,
                writer_args=self._writer_args,
                version=self._version,
            )

        def _load(self) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
            load_path = get_filepath_str(self._get_load_path(), self._protocol)

            with self._fs.open(load_path, **self._fs_open_args_load) as fs_file:
                return pd.read_excel(fs_file, **self._load_args)

        def _save(self, data: Union[pd.DataFrame, Dict[str, pd.DataFrame]]) -> None:
            output = BytesIO()
            save_path = get_filepath_str(self._get_save_path(), self._protocol)

            # pylint: disable=abstract-class-instantiated
            with pd.ExcelWriter(output, **self._writer_args) as writer:
                if isinstance(data, dict):
                    for sheet_name, sheet_data in data.items():
                        sheet_data.to_excel(
                            writer, sheet_name=sheet_name, **self._save_args
                        )
                else:
                    data.to_excel(writer, **self._save_args)

            with self._fs.open(save_path, **self._fs_open_args_save) as fs_file:
                fs_file.write(output.getvalue())

            self._invalidate_cache()

        def _exists(self) -> bool:
            try:
                load_path = get_filepath_str(self._get_load_path(), self._protocol)
            except DataSetError:
                return False

            return self._fs.exists(load_path)

        def _release(self) -> None:
            super()._release()
            self._invalidate_cache()

        def _invalidate_cache(self) -> None:
            """Invalidate underlying filesystem caches."""
            filepath = get_filepath_str(self._filepath, self._protocol)
            self._fs.invalidate_cache(filepath)